@@ -34,6 +34,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <math.h>
 #include <ctype.h>
 #include <dirent.h>
+#include <locale.h>
 
 #include <glib.h>
 #include <glib/gi18n.h>
@@ -44,6 +45,8 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 /* Variable and macro definitions */
 /*---------------------------------------------------------------------------*/
 
+#define SUDO_PREFIX "SUDO_ASKPASS=/usr/bin/sudopwd sudo -A "
+
 /* struct to store partition data */
 
 #define MAXPART 9
@@ -133,15 +136,18 @@ static int get_string (char *cmd, char *name)
 
 static int sys_printf (const char * format, ...)
 {
-    char buffer[256];
+    char *buffer;
     va_list args;
     FILE *fp;
+    int res;
 
     va_start (args, format);
-    vsprintf (buffer, format, args);
-    fp = popen (buffer, "r");
+    g_vasprintf (&buffer, format, args);
     va_end (args);
-    return pclose (fp);
+    fp = popen (buffer, "r");
+    res = pclose (fp);
+    g_free (buffer);
+    return res;
 }
 
 
@@ -152,7 +158,7 @@ static int sys_printf (const char * format, ...)
 
 static gboolean close_msg (gpointer data)
 {
-    gtk_widget_destroy (GTK_WIDGET (msg_dlg));
+    gtk_widget_destroy (msg_dlg);
     return FALSE;
 }
 
@@ -191,17 +197,13 @@ static char *partition_name (char *device, char *buffer)
 
 static gboolean cb_update_progress (gpointer data)
 {
-    float *fptr = (float *) &data;
-    gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progress), *fptr);
+    gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progress), *((float *) data));
     return FALSE;
 }
 
 static void update_progress (float prog)
 {
-    void *ptr;
-    float *fptr = (float *) &ptr;
-    *fptr = prog;
-    gdk_threads_add_idle (cb_update_progress, ptr);
+    gdk_threads_add_idle (cb_update_progress, (void *) &prog);
 }
 
 static gboolean cb_update_label (gpointer data)
@@ -223,7 +225,7 @@ static void update_label (char *msg)
 static gpointer copy_thread (gpointer data)
 {
     copying = 1;
-    sys_printf ("cp -ax %s/. %s/.", src_mnt, dst_mnt);
+    sys_printf (SUDO_PREFIX "cp -ax %s/. %s/.", src_mnt, dst_mnt);
     copying = 0;
     return NULL;
 }
@@ -243,7 +245,7 @@ static gpointer backup_thread (gpointer data)
     get_string ("uuid | cut -f1 -d-", npuuid);
 
     // check the source has an msdos partition table
-    sprintf (buffer, "parted %s unit s print | tail -n +4 | head -n 1", src_dev);
+    sprintf (buffer, SUDO_PREFIX "parted %s unit s print | tail -n +4 | head -n 1", src_dev);
     fp = popen (buffer, "r");
     if (fp == NULL) return NULL;
     if (fgets (buffer, sizeof (buffer) - 1, fp) == NULL)
@@ -268,12 +270,12 @@ static gpointer backup_thread (gpointer data)
     // unmount any partitions on the target device
     for (n = 9; n >= 1; n--)
     {
-        sys_printf ("umount %s%d", partition_name (dst_dev, dev), n);
+        sys_printf (SUDO_PREFIX "umount %s%d", partition_name (dst_dev, dev), n);
         CANCEL_CHECK;
     }
 
     // wipe the FAT on the target
-    if (sys_printf ("dd if=/dev/zero of=%s bs=512 count=1", dst_dev))
+    if (sys_printf (SUDO_PREFIX "dd if=/dev/zero of=%s bs=512 count=1", dst_dev))
     {
         terminate_dialog (_("Could not write to destination."));
         return NULL;
@@ -287,7 +289,7 @@ static gpointer backup_thread (gpointer data)
     CANCEL_CHECK;
     
     // prepare the new FAT
-    if (sys_printf ("parted -s %s mklabel msdos", dst_dev))
+    if (sys_printf (SUDO_PREFIX "parted -s %s mklabel msdos", dst_dev))
     {
         terminate_dialog (_("Could not create FAT."));
         return NULL;
@@ -300,7 +302,7 @@ static gpointer backup_thread (gpointer data)
 
     // read in the source partition table
     n = 0;
-    sprintf (buffer, "parted %s unit s print | sed '/^ /!d'", src_dev);
+    sprintf (buffer, SUDO_PREFIX "parted %s unit s print | sed '/^ /!d'", src_dev);
     fp = popen (buffer, "r");
     if (fp != NULL)
     {
@@ -331,7 +333,7 @@ static gpointer backup_thread (gpointer data)
         // create the partition
         if (!strcmp (parts[p].ptype, "extended"))
         {
-            if (sys_printf ("parted -s %s -- mkpart extended %lds -1s", dst_dev, parts[p].start))
+            if (sys_printf (SUDO_PREFIX "parted -s %s -- mkpart extended %lds -1s", dst_dev, parts[p].start))
             {
                 terminate_dialog (_("Could not create partition."));
                 return NULL;
@@ -341,7 +343,7 @@ static gpointer backup_thread (gpointer data)
         {
             if (p == (n - 1))
             {
-                if (sys_printf ("parted -s %s -- mkpart %s %s %lds -1s", dst_dev,
+                if (sys_printf (SUDO_PREFIX "parted -s %s -- mkpart %s %s %lds -1s", dst_dev,
                     parts[p].ptype, parts[p].ftype, parts[p].start))
                 {
                     terminate_dialog (_("Could not create partition."));
@@ -350,7 +352,7 @@ static gpointer backup_thread (gpointer data)
             }
             else
             {
-                if (sys_printf ("parted -s %s mkpart %s %s %lds %lds", dst_dev,
+                if (sys_printf (SUDO_PREFIX "parted -s %s mkpart %s %s %lds %lds", dst_dev,
                     parts[p].ptype, parts[p].ftype, parts[p].start, parts[p].end))
                 {
                     terminate_dialog (_("Could not create partition."));
@@ -361,7 +363,7 @@ static gpointer backup_thread (gpointer data)
         CANCEL_CHECK;
 
         // refresh the kernel partion table
-        sys_printf ("partprobe");
+        sys_printf (SUDO_PREFIX "partprobe");
         CANCEL_CHECK;
 
         // get the UUID
@@ -400,22 +402,22 @@ static gpointer backup_thread (gpointer data)
         if (!strlen (res)) lbl = 0;
 
         // get the partition UUID
-        sprintf (buffer, "blkid %s | rev | cut -f 2 -d ' ' | rev | cut -f 2 -d \\\"", src_dev);
+        sprintf (buffer, SUDO_PREFIX "blkid %s | rev | cut -f 2 -d ' ' | rev | cut -f 2 -d \\\"", src_dev);
         puid = get_string (buffer, puuid);
         if (!strlen (puuid)) puid = 0;
 
         // create file systems
         if (!strncmp (parts[p].ftype, "fat", 3))
         {
-            if (uid) sprintf (buffer, "mkfs.fat -F 32 -i %s %s%d", uuid, partition_name (dst_dev, dev), parts[p].pnum);
-            else sprintf (buffer, "mkfs.fat -F 32 %s%d", partition_name (dst_dev, dev), parts[p].pnum);
+            if (uid) sprintf (buffer, SUDO_PREFIX "mkfs.fat -F 32 -i %s %s%d", uuid, partition_name (dst_dev, dev), parts[p].pnum);
+            else sprintf (buffer, SUDO_PREFIX "mkfs.fat -F 32 %s%d", partition_name (dst_dev, dev), parts[p].pnum);
 
             if (sys_printf (buffer))
             {
                 if (uid)
                 {
                     // second try just in case the only problem was a corrupt UUID
-                    sprintf (buffer, "mkfs.fat -F 32 %s%d", partition_name (dst_dev, dev), parts[p].pnum);
+                    sprintf (buffer, SUDO_PREFIX "mkfs.fat -F 32 %s%d", partition_name (dst_dev, dev), parts[p].pnum);
                     if (sys_printf (buffer))
                     {
                         terminate_dialog (_("Could not create file system."));
@@ -429,21 +431,21 @@ static gpointer backup_thread (gpointer data)
                 }
             }
 
-            if (lbl) sys_printf ("fatlabel %s%d %s", partition_name (dst_dev, dev), parts[p].pnum, res);
+            if (lbl) sys_printf (SUDO_PREFIX "fatlabel %s%d %s", partition_name (dst_dev, dev), parts[p].pnum, res);
         }
         CANCEL_CHECK;
 
         if (!strcmp (parts[p].ftype, "ext4"))
         {
-            if (uid) sprintf (buffer, "mkfs.ext4 -F -U %s %s%d", uuid, partition_name (dst_dev, dev), parts[p].pnum);
-            else sprintf (buffer, "mkfs.ext4 -F %s%d", partition_name (dst_dev, dev), parts[p].pnum);
+            if (uid) sprintf (buffer, SUDO_PREFIX "mkfs.ext4 -F -U %s %s%d", uuid, partition_name (dst_dev, dev), parts[p].pnum);
+            else sprintf (buffer, SUDO_PREFIX "mkfs.ext4 -F %s%d", partition_name (dst_dev, dev), parts[p].pnum);
 
             if (sys_printf (buffer))
             {
                 if (uid)
                 {
                     // second try just in case the only problem was a corrupt UUID
-                    sprintf (buffer, "mkfs.ext4 -F %s%d", partition_name (dst_dev, dev), parts[p].pnum);
+                    sprintf (buffer, SUDO_PREFIX "mkfs.ext4 -F %s%d", partition_name (dst_dev, dev), parts[p].pnum);
                     if (sys_printf (buffer))
                     {
                         terminate_dialog (_("Could not create file system."));
@@ -457,12 +459,12 @@ static gpointer backup_thread (gpointer data)
                 }
             }
 
-            if (lbl) sys_printf ("e2label %s%d %s", partition_name (dst_dev, dev), parts[p].pnum, res);
+            if (lbl) sys_printf (SUDO_PREFIX "e2label %s%d %s", partition_name (dst_dev, dev), parts[p].pnum, res);
         }
         CANCEL_CHECK;
 
         // write the partition UUID
-        if (puid) sys_printf ("echo \"x\ni\n0x%s\nr\nw\n\" | fdisk %s", new_uuid ? npuuid : puuid, dst_dev);
+        if (puid) sys_printf ("echo \"x\ni\n0x%s\nr\nw\n\" | %sfdisk %s", new_uuid ? npuuid : puuid, SUDO_PREFIX, dst_dev);
         CANCEL_CHECK;
 
         prog = p + 1;
@@ -484,13 +486,13 @@ static gpointer backup_thread (gpointer data)
             get_string ("partprobe", res);
 
             // mount partitions
-            if (sys_printf ("mount %s%d %s", partition_name (dst_dev, dev), parts[p].pnum, dst_mnt))
+            if (sys_printf (SUDO_PREFIX "mount %s%d %s", partition_name (dst_dev, dev), parts[p].pnum, dst_mnt))
             {
                 terminate_dialog (_("Could not mount partition."));
                 return NULL;
             }
             CANCEL_CHECK;
-            if (sys_printf ("mount %s%d %s", partition_name (src_dev, dev), parts[p].pnum, src_mnt))
+            if (sys_printf (SUDO_PREFIX "mount %s%d %s", partition_name (src_dev, dev), parts[p].pnum, src_mnt))
             {
                 terminate_dialog (_("Could not mount partition."));
                 return NULL;
@@ -508,8 +510,8 @@ static gpointer backup_thread (gpointer data)
 
             if (srcsz >= dstsz)
             {
-                sys_printf ("umount %s", dst_mnt);
-                sys_printf ("umount %s", src_mnt);
+                sys_printf (SUDO_PREFIX "umount %s", dst_mnt);
+                sys_printf (SUDO_PREFIX "umount %s", src_mnt);
                 terminate_dialog (_("Insufficient space. Backup aborted."));
                 return NULL;
             }
@@ -518,7 +520,7 @@ static gpointer backup_thread (gpointer data)
             g_thread_new (NULL, copy_thread, NULL);
 
             // get the size to be copied
-            sprintf (buffer, "du -s %s", src_mnt);
+            sprintf (buffer, SUDO_PREFIX "du -s %s", src_mnt);
             get_string (buffer, res);
             sscanf (res, "%ld", &srcsz);
             if (srcsz < 50000) stime = 1;
@@ -526,7 +528,7 @@ static gpointer backup_thread (gpointer data)
             else stime = 10;
 
             // wait for the copy to complete, while updating the progress bar...
-            sprintf (buffer, "du -s %s", dst_mnt);
+            sprintf (buffer, SUDO_PREFIX "du -s %s", dst_mnt);
             while (copying)
             {
                 get_string (buffer, res);
@@ -544,18 +546,18 @@ static gpointer backup_thread (gpointer data)
             if (puid && new_uuid)
             {
                 // relevant files are dst_mnt/etc/fstab and dst_mnt/boot/cmdline.txt
-                sys_printf ("if [ -e /%s/etc/fstab ] ; then sed -i s/%s/%s/g /%s/etc/fstab ; fi", dst_mnt, puuid, npuuid, dst_mnt);
-                sys_printf ("if [ -e /%s/cmdline.txt ] ; then sed -i s/%s/%s/g /%s/cmdline.txt ; fi", dst_mnt, puuid, npuuid, dst_mnt);
+                sys_printf ("if [ -e /%s/etc/fstab ] ; then %ssed -i s/%s/%s/g /%s/etc/fstab ; fi", dst_mnt, SUDO_PREFIX, puuid, npuuid, dst_mnt);
+                sys_printf ("if [ -e /%s/cmdline.txt ] ; then %ssed -i s/%s/%s/g /%s/cmdline.txt ; fi", dst_mnt, SUDO_PREFIX, puuid, npuuid, dst_mnt);
             }
 
             // unmount partitions
-            if (sys_printf ("umount %s", dst_mnt))
+            if (sys_printf (SUDO_PREFIX "umount %s", dst_mnt))
             {
                 terminate_dialog (_("Could not unmount partition."));
                 return NULL;
             }
             CANCEL_CHECK;
-            if (sys_printf ("umount %s", src_mnt))
+            if (sys_printf (SUDO_PREFIX "umount %s", src_mnt))
             {
                 terminate_dialog (_("Could not unmount partition."));
                 return NULL;
@@ -566,7 +568,7 @@ static gpointer backup_thread (gpointer data)
         // set the flags
         if (!strcmp (parts[p].flags, "lba"))
         {
-            if (sys_printf ("parted -s %s set %d lba on", dst_dev, parts[p].pnum))
+            if (sys_printf (SUDO_PREFIX "parted -s %s set %d lba on", dst_dev, parts[p].pnum))
             {
                 terminate_dialog (_("Could not set flags."));
                 return NULL;
@@ -574,7 +576,7 @@ static gpointer backup_thread (gpointer data)
         }
         else
         {
-            if (sys_printf ("parted -s %s set %d lba off", dst_dev, parts[p].pnum))
+            if (sys_printf (SUDO_PREFIX "parted -s %s set %d lba off", dst_dev, parts[p].pnum))
             {
                 terminate_dialog (_("Could not set flags."));
                 return NULL;
@@ -583,6 +585,10 @@ static gpointer backup_thread (gpointer data)
         CANCEL_CHECK;
     }
 
+    if (getenv ("WAYLAND_DISPLAY"))
+        sys_printf ("wfpanelctl ejecter %s", dst_dev);
+    else
+        sys_printf ("lxpanelctl command ejecter %s", dst_dev);
     terminate_dialog (_("Copy complete."));
     return NULL;
 }
@@ -606,7 +612,7 @@ static void kill_copy (void)
             while (1)
             {
                 if (fgets (buffer, sizeof (buffer) - 1, fp) == NULL) break;
-                if (sscanf (buffer, "%d", &pid) == 1) sys_printf ("kill %d", pid);
+                if (sscanf (buffer, "%d", &pid) == 1) sys_printf (SUDO_PREFIX "kill %d", pid);
             }
             pclose (fp);
         }
@@ -676,7 +682,7 @@ static gboolean on_start (void)
     wid = (GtkWidget *) gtk_builder_get_object (builder, "modal_ok");
     gtk_widget_hide (wid);
 
-    gtk_widget_show (GTK_WIDGET (msg_dlg));
+    gtk_widget_show (msg_dlg);
 
     // launch a thread with the system call to run the backup
     cancelled = 0;
@@ -702,7 +708,7 @@ static gboolean on_close (void)
 
 static gboolean on_confirm (void)
 {
-    char buffer[256], res[256];
+    char buffer[256];
     char *src, *dst;
     int len;
     GtkBuilder *builder;
@@ -750,7 +756,7 @@ static gboolean on_confirm (void)
     g_free (src);
     g_free (dst);
 
-    gtk_widget_show (GTK_WIDGET (msg_dlg));
+    gtk_widget_show (msg_dlg);
     state = STATE_CONF;
     return FALSE;
 }
@@ -804,7 +810,6 @@ static void on_drives_changed (GVolumeMonitor *volume_monitor, GDrive *drive, gp
 {
     char buffer[256], test[128];
     FILE *fp;
-    int devlen;
 
     if (state == STATE_CONF)
     {
@@ -881,12 +886,10 @@ int main (int argc, char *argv[])
 {
     GtkBuilder *builder;
 
-#ifdef ENABLE_NLS
     setlocale (LC_ALL, "");
     bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
     bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
     textdomain (GETTEXT_PACKAGE);
-#endif
 
     // GTK setup
     gtk_init (&argc, &argv);
